% % % % % % % % Model Initialization % % % % % % % % 
clear

bspline

%% Environment
env = struct;
env.ISA_lapse = 0.0065;
env.ISA_hmax = 2000;
env.ISA_R = 287.0531;
env.ISA_g = double(9.80665);
env.ISA_rho0 = 1.225;
env.ISA_P0 = 101325;
env.ISA_T0 = 288.15;
env.windBase = 12;
env.windDirTurb = 180;
env.windDirHor = 90;
env.windOn = 0;

%% Sensors
sensors = struct;
sensors.noiseOn = 0;
sensors.max_euler = 6.2831853071795862;
sensors.max_accel = 4;
sensors.bit_scale = 32768;
sensors.max_gyro = 4;
sensors.max_temp = 150;
sensors.max_dyn = 1800;
sensors.gpsRefreshRate = 0.2;
sensors.max_ctrlSrfc = 1.5707963267948966;
sensors.antialiasBW = 40;

%% UAV
uav = struct;
uav.geometry = struct;
uav.geometry.span = 1.2;   % wing span, parameter b
uav.geometry.chord = 0.3;  % wing chord, parameter c 
uav.geometry.S = 0.32;     % wing area
uav.geometry.AR = 4.5;     % aspect ratio b^2/S
uav.geometry.mass = 1.9;   % Fixed wing UAV quality, parameter m
uav.inertia = struct;
uav.inertia.Ixx = 0.0894;  % x aixs, principal moment of inertia
uav.inertia.Iyy = 0.144;   % y aixs, principal moment of inertia
uav.inertia.Izz = 0.162;   % z aixs, principal moment of inertia
uav.inertia.Ixz = 0.014;   % product of inertia
uav.aero = struct;
% Lift coefficient
uav.aero.CL0 = 0.23;       
uav.aero.CLa = 4.58;
uav.aero.CLa_dot = 1.97;
uav.aero.CLq = 7.95;
uav.aero.CLDe = 0.13;
uav.aero.CLDf = 0;
uav.aero.CLmin = 0.23;
% Drag coefficient
uav.aero.CD0 = 0.0434;
uav.aero.CDDe = 0.0135;
uav.aero.CDDr = 0.0303;
uav.aero.e = 0.7;          % span efficiency factor 
% Yawing force coefficient
uav.aero.CYb = -0.83;
uav.aero.CYDr = 0.191;
% Yaw moment coefficient
uav.aero.Clb = -0.04;
uav.aero.Clp = -0.414;
uav.aero.Clr = 0.399;
uav.aero.ClDa = 0.0677;
uav.aero.ClDr = 0.0168;
% Pitch moment coefficient
uav.aero.Cm0 = 0.135;
uav.aero.Cma = -1.5;
uav.aero.Cma_dot = -10.4;
uav.aero.Cmq = -50.8;
uav.aero.CmDe = -1.13;
uav.aero.CmDf = 0;
% Roll moment coefficient
uav.aero.Cnb = 0.0344;
uav.aero.Cnp = -0.075;
uav.aero.Cnr = -0.411;
uav.aero.CnDa = -0.012;
uav.aero.CnDr = -0.0345;
% engine model coefficient
uav.engine = struct;
uav.engine.MinThK = 0.077;
uav.engine.ThK = 2.17;
uav.engine.TFact = 0.2;
% motor 
uav.prop = struct;
uav.prop.Dp = 0.3048;       % propeller diameter
uav.prop.Ct = 0.0931;      % tension coefficient
uav.prop.Kv = 450;         
uav.prop.U = 22.2;         % maximum motor voltage
uav.prop.decay = 1.0697;        % motor attenuation coefficient
% Steering engine deflection limit
uav.elevator.max = 20/180*pi;
uav.elevator.min = -20/180*pi;
uav.aileron.max = 23/180*pi;
uav.aileron.min = -23/180*pi;
uav.rudder.max = 25/180*pi;
uav.rudder.min = -25/180*pi;
uav.flap.max = 10/180*pi;
uav.flap.min = -10/180*pi;
uav.throttlepercent.max = 1;
uav.throttlepercent.min = 0;
uav.controlsurfacemax = 90/180*pi;
% Attitude change limit
uav.attitudectrlmax.roll = 23/180*pi;
uav.attitudectrlmax.pitch = 20/180*pi;
uav.attitudectrlmax.yawrate = 2/180*pi;
% Atitude change limit
uav.altitude.max = 100;
% Airspeed change limit
uav.speed.max = 40;
uav.attitudectrlmax.tpercent = 1;


%% Initial Condition
% Line test
% uav.ic = struct;
% uav.ic.Pos_0 = [0; 0; 50];
% uav.ic.Euler_0 = [0; 0; pi/4];
% uav.ic.Omega_0 = [0; 0; 0];
% uav.ic.PQR_0 = [0; 0; 0];
% uav.ic.Vb_0 = [20; 0; 0];
% uav.ic.gsLL = [45 120];
% uav.ic.gsH = 0;

% Circle test
% uav.ic = struct;
% uav.ic.Pos_0 = [0; 0; 50];
% uav.ic.Euler_0 = [0; 0; 0];
% uav.ic.Omega_0 = [0; 0; 0];
% uav.ic.PQR_0 = [0; 0; 0];
% uav.ic.Vb_0 = [20; 0; 0];
% uav.ic.gsLL = [45 120];
% uav.ic.gsH = 0;

%Path 
uav.ic = struct;
uav.ic.Pos_0 = [xt(1); -yt(1); double(100)];
uav.ic.Euler_0 = [0; 0; 0];
uav.ic.Omega_0 = [0; 0; 0];
uav.ic.PQR_0 = [0; 0; 0];
uav.ic.Vb_0 = [3; 0; 0];
uav.ic.gsLL = [45 120];
uav.ic.gsH = 0;

%% Sample Time
uav.sampleTime = 0.005;
WindSampleTime = 0.01;
SenorSampleTime = 0.01;