% % % % % % % % Model Initialization % % % % % % % % 

bspline



%% Environment
env = struct;
env.ISA_lapse = 0.0065;
env.ISA_hmax = 2000;
env.ISA_R = 287.0531;
env.ISA_g = double(9.80665);
env.ISA_rho0 = 1.225;
env.ISA_P0 = 101325;
env.ISA_T0 = 288.15;
env.windBase = 12;
env.windDirTurb = 180;
env.windDirHor = 90;
env.windOn = 0;

%% Sensors
sensors = struct;
sensors.noiseOn = 0;
sensors.max_euler = 6.2831853071795862;
sensors.max_accel = 4;
sensors.bit_scale = 32768;
sensors.max_gyro = 4;
sensors.max_temp = 150;
sensors.max_dyn = 1800;
sensors.gpsRefreshRate = 0.2;
sensors.max_ctrlSrfc = 1.5707963267948966;
sensors.antialiasBW = 40;

%% UAV
uav = struct;
uav.geometry = struct;
uav.geometry.span = 2.795;
uav.geometry.chord = 0.35134168157423973;
uav.geometry.S = 0.982;
uav.geometry.elarm = 1.21;
uav.geometry.mass = 8.1646626600000012;
uav.inertia = struct;
uav.inertia.Ixx = 4.12;
uav.inertia.Iyy = 9.58;
uav.inertia.Izz = 9.85;
uav.aero = struct;
uav.aero.CL0 = 0.38;
uav.aero.CLa = 18.5;
uav.aero.CLa_dot = 2.64;
uav.aero.CLq = 7.4;
uav.aero.CLDe = 0.24;
uav.aero.CLDf = 0.4;
uav.aero.CD0 = 0.022;
uav.aero.A1 = 0.007;
uav.aero.Apolar = 0.057;
uav.aero.CYb = -1.098;
uav.aero.CYDr = 0.143;
uav.aero.Clb = -0.296;
uav.aero.Clp = -1.96;
uav.aero.Clr = 0.103;
uav.aero.ClDa = 0.1695;
uav.aero.ClDr = 0.106;
uav.aero.Cm0 = 0.3;
uav.aero.Cma = -1.239;
uav.aero.Cma_dot = -7;
uav.aero.Cmq = -2.4;
uav.aero.CmDe = -3.2;
uav.aero.CmDf = -0.021;
uav.aero.Cnb = 0.277;
uav.aero.Cnp = -0.0889;
uav.aero.Cnr = -0.19997;
uav.aero.CnDa = -0.023;
uav.aero.CnDr = -0.1997;
uav.engine = struct;
uav.engine.MinThK = 0.077;
uav.engine.ThK = 2.17;
uav.engine.TFact = 0.2;
uav.elevator.max = 30/180*pi;
uav.elevator.min = -30/180*pi;
uav.aileron.max = 30/180*pi;
uav.aileron.min = -30/180*pi;
uav.rudder.max = 30/180*pi;
uav.rudder.min = -30/180*pi;
uav.flap.max = 10/180*pi;
uav.flap.min = -10/180*pi;
uav.controlsurfacemax = 90/180*pi;
uav.throttlepercent.max = 1;
uav.throttlepercent.min = 0;
uav.attitudectrlmax.roll = 20/180*pi;
uav.attitudectrlmax.pitch = 20/180*pi;
uav.attitudectrlmax.yawrate = 2/180*pi;
uav.attitudectrlmax.tpercent = 1;
uav.speed.max = 20;
uav.altitude.max = 100;

%% Initial Condition
% Line test
% uav.ic = struct;
% uav.ic.Pos_0 = [0; 0; 50];
% uav.ic.Euler_0 = [0; 0; pi/4];
% uav.ic.Omega_0 = [0; 0; 0];
% uav.ic.PQR_0 = [0; 0; 0];
% uav.ic.Vb_0 = [20; 0; 0];
% uav.ic.gsLL = [45 120];
% uav.ic.gsH = 0;

% Circle test
% uav.ic = struct;
% uav.ic.Pos_0 = [0; 0; 50];
% uav.ic.Euler_0 = [0; 0; 0];
% uav.ic.Omega_0 = [0; 0; 0];
% uav.ic.PQR_0 = [0; 0; 0];
% uav.ic.Vb_0 = [20; 0; 0];
% uav.ic.gsLL = [45 120];
% uav.ic.gsH = 0;

%Path 
uav.ic = struct;
uav.ic.Pos_0 = [xt(1); -yt(1); -50];
uav.ic.Euler_0 = [0; 0; 0];
uav.ic.Omega_0 = [0; 0; 0];
uav.ic.PQR_0 = [0; 0; 0];
uav.ic.Vb_0 = [3; 0; 0];
uav.ic.gsLL = [45 120];
uav.ic.gsH = 0;

%% Sample Time
uav.sampleTime = 0.005;
WindSampleTime = 0.01;
SenorSampleTime = 0.01;